import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.webhook_request import WebhookRequest
from async_yookassa.models.webhook_response import WebhookList, WebhookResponse
from async_yookassa.utils import get_base_headers


class Webhook:
    """
    Класс, представляющий модель Webhook.
    """

    base_path = "/webhooks"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def list(cls) -> WebhookList:
        """
        Возвращает список вебхуков

        :return: Объект ответа WebhookList, возвращаемого API при запросе списка вебхуков
        """
        instance = cls()

        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)

        return WebhookList(**response)

    @classmethod
    async def add(
        cls, params: dict[str, Any] | WebhookRequest, idempotency_key: uuid.UUID | None = None
    ) -> WebhookResponse:
        """
        Добавление вебхука

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа WebhookResponse, возвращаемого API при запросе информации о вебхуке
        """
        instance = cls()

        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = WebhookRequest(**params)
        elif isinstance(params, WebhookRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )

        return WebhookResponse(**response)

    @classmethod
    async def remove(cls, webhook_id: str, idempotency_key: uuid.UUID | None = None) -> WebhookResponse:
        """
        Удаление вебхука

        :param webhook_id: Уникальный идентификатор вебхука
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа WebhookResponse, возвращаемого API при запросе информации о вебхуке
        """
        instance = cls()

        path = cls.base_path + "/" + webhook_id

        headers = get_base_headers(idempotency_key=idempotency_key)

        response = await instance.client.request(method=HTTPMethodEnum.DELETE, path=path, headers=headers)

        return WebhookResponse(**response)



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount import Amount


class ReceiptData(BaseModel):
    service_name: str = Field(min_length=1, max_length=50)
    amount: Amount | None = None


class ReceiptDataResponse(ReceiptData):
    npd_receipt_id: str | None = None
    url: str | None = None



import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.payout_request import PayoutRequest
from async_yookassa.models.payout_response import PayoutResponse
from async_yookassa.utils import get_base_headers


class Payout:
    """
    Класс, представляющий модель Payout.
    """

    base_path = "/payouts"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def find_one(cls, payout_id: str) -> PayoutResponse:
        """
        Возвращает информацию о выплате

        :param payout_id: Уникальный идентификатор выплаты
        :return: Объект ответа PayoutResponse, возвращаемого API при запросе выплаты
        """
        instance = cls()

        if not isinstance(payout_id, str):
            raise ValueError("Invalid payout_id value")

        path = instance.base_path + "/" + payout_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)

        return PayoutResponse(**response)

    @classmethod
    async def create(
        cls, params: dict[str, Any] | PayoutRequest, idempotency_key: uuid.UUID | None = None
    ) -> PayoutResponse:
        """
        Создание выплаты

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа PayoutResponse, возвращаемого API при запросе выплате
        """
        instance = cls()
        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = PayoutRequest(**params)
        elif isinstance(params, PayoutRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )
        return PayoutResponse(**response)



from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.me import Me


class Settings:
    """
    Класс, представляющий модель Settings.
    """

    base_path = "/me"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def get_account_settings(cls, params: dict[str, str] | None = None) -> Me:
        """
        Возвращает информацию о магазине

        :param params: (dict | None) Параметры поиска.
            В настоящее время доступен только {'on_behalf_of': account_id}
        :return: Me Информация о настройках магазина или шлюза
        """
        instance = cls()
        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path, query_params=params)
        return Me(**response)



from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.deal_enums import DealFeeMomentEnum, DealTypeEnum


class DealRequest(BaseModel):
    type: DealTypeEnum
    fee_moment: DealFeeMomentEnum
    metadata: dict[str, Any] | None = None
    description: str | None = Field(max_length=128, default=None)

    model_config = ConfigDict(use_enum_values=True)



from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.payment_response import ReceiptRegistrationEnum
from async_yookassa.models.invoice_submodels.cart import Cart
from async_yookassa.models.invoice_submodels.delivery_method import DeliveryMethod
from async_yookassa.models.invoice_submodels.invoice_cancellation_details import (
    InvoiceCancellationDetails,
)
from async_yookassa.models.invoice_submodels.payment_details import PaymentDetails


class InvoiceResponse(BaseModel):
    id: str = Field(min_length=39, max_length=39)
    status: ReceiptRegistrationEnum
    cart: list[Cart]
    delivery_method: DeliveryMethod | None = None
    payment_details: PaymentDetails | None = None
    created_at: datetime
    expires_at: datetime | None = None
    description: str | None = Field(max_length=128, default=None)
    cancellation_details: InvoiceCancellationDetails | None = None
    metadata: dict[str, Any] | None = None

    model_config = ConfigDict(use_enum_values=True)



import re

from pydantic import BaseModel, field_validator
from pydantic.v1 import Field

from async_yookassa.models.payment_submodels.airline_submodels.legs import Leg
from async_yookassa.models.payment_submodels.airline_submodels.passengers import (
    Passenger,
)


class Airline(BaseModel):
    ticket_number: str | None = Field(max_length=150, default=None)
    booking_reference: str | None = Field(max_length=20, default=None)
    passengers: list[Passenger] | None = None
    legs: list[Leg] | None = None

    @field_validator("ticket_number", mode="before")
    def ticket_number_validator(cls, value: str) -> str:
        """
        Устанавливает ticket_number модели Airline.

        :param value: ticket_number модели Airline.
        :type value: str
        """
        if not re.match("^[0-9]{1,150}$", value):
            raise ValueError(
                r"Invalid value for `ticket_number`, must be a follow pattern or equal to `/^[0-9]{1,150}$/`"
            )

        return value

    @field_validator("passengers", mode="before")
    def passengers_validator(cls, value: list[Passenger]) -> list[Passenger]:
        """
        Устанавливает passengers модели Airline.

        :param value: passengers модели Airline.
        :type value: list[Passenger]
        """
        if len(value) > 500:
            raise ValueError("Invalid value for `passengers`, number of items must be less than or equal to `500`")

        return value

    @field_validator("legs", mode="before")
    def legs_validator(cls, value: list[Leg]) -> list[Leg]:
        """
        Устанавливает legs модели Airline.

        :param value: legs модели Airline.
        :type value: list[Leg]
        """
        if len(value) > 4:
            raise ValueError("Invalid value for `legs`, number of items must be less than or equal to `4`")

        return value



from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, model_validator

from async_yookassa.enums.personal_data_enums import PersonalDataTypeEnum


class PersonalDataRequest(BaseModel):
    type: PersonalDataTypeEnum
    last_name: str
    first_name: str
    middle_name: str | None = None
    birthdate: datetime | None = None
    metadata: dict[str, Any] | None = None

    model_config = ConfigDict(use_enum_values=True)

    @model_validator(mode="before")
    def validate_required_fields(cls, values: dict[str, Any]) -> dict[str, Any]:
        type_value = values.get("type")

        if type_value == PersonalDataTypeEnum.payout_statement_recipient:
            if not values.get("birthdate"):
                raise ValueError("Field 'birthdate' is required for type 'payout_statement_recipient'")

        return values



from enum import Enum


class ConfirmationTypeEnum(str, Enum):
    embedded = "embedded"
    external = "external"
    mobile_application = "mobile_application"
    qr = "qr"
    redirect = "redirect"



from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field

from async_yookassa.enums.payment_response import ReceiptRegistrationEnum
from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.cancellation_details import PayoutDetails
from async_yookassa.models.payment_submodels.deal import DealBase
from async_yookassa.models.payout_submodels.payout_destination_data import (
    PayoutDestinationResponse,
)
from async_yookassa.models.payout_submodels.receipt_data import ReceiptDataResponse


class PayoutResponse(BaseModel):
    id: str = Field(min_length=36, max_length=50)
    amount: Amount
    status: ReceiptRegistrationEnum
    payout_destination: PayoutDestinationResponse
    description: str | None = Field(max_length=128, default=None)
    created_at: datetime
    deal: DealBase | None = None
    self_employed: DealBase | None = None
    receipt: ReceiptDataResponse | None = None
    cancellation_details: PayoutDetails | None = None
    metadata: dict[str, Any] | None = None
    test: bool



from pydantic import BaseModel

from async_yookassa.models.webhook_request import WebhookRequest


class WebhookResponse(WebhookRequest):
    id: str


class WebhookList(BaseModel):
    type: str
    items: list[WebhookResponse]
    next_cursor: str | None = None



from pydantic import BaseModel, ConfigDict, Field, model_validator

from async_yookassa.enums.payment_method_data import PaymentMethodDataTypeEnum
from async_yookassa.models.payment_submodels.payment_method_submodels.articles import (
    Article,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.card import (
    CardRequest,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.electronic_certificate import (
    ElectronicCertificate,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.vat_data import (
    VatData,
)


class PaymentMethodData(BaseModel):
    type: PaymentMethodDataTypeEnum
    phone: str | None = None
    card: CardRequest | None = None
    payment_purpose: str | None = Field(max_length=210, default=None)
    vat_data: VatData | None = None
    articles: list[Article] | None = None
    electronic_certificate: ElectronicCertificate | None = None

    model_config = ConfigDict(use_enum_values=True)

    @model_validator(mode="before")
    def validate_required_fields(cls, values):
        type_value = values.get("type")

        if type_value == PaymentMethodDataTypeEnum.mobile_balance:
            if not values.get("phone"):
                raise ValueError("Field 'phone' is required for type 'mobile_balance'")

        elif type_value == PaymentMethodDataTypeEnum.b2b_sberbank:
            if not values.get("payment_purpose"):
                raise ValueError("Field 'payment_purpose' are required for type 'b2b_sberbank'")
            if not values.get("vat_data"):
                raise ValueError("Field 'vat_data' are required for type 'b2b_sberbank'")

        return values



from pydantic import BaseModel, Field


class SBPPayerBankDetails(BaseModel):
    bank_id: str = Field(max_length=12)
    bic: str


class B2BSBPayerBankDetails(BaseModel):
    full_name: str = Field(max_length=800)
    short_name: str = Field(max_length=160)
    address: str = Field(max_length=500)
    inn: str = Field(max_length=12)
    bank_name: str = Field(min_length=1, max_length=350)
    bank_branch: str = Field(min_length=1, max_length=140)
    bank_bik: str = Field(max_length=9)
    account: str = Field(max_length=20)
    kpp: str | None = Field(max_length=9, default=None)



from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.personal_data_enums import (
    PersonalDataStatusEnum,
    PersonalDataTypeEnum,
)
from async_yookassa.models.payment_submodels.cancellation_details import (
    PersonalDataDetails,
)


class PersonalDataResponse(BaseModel):
    id: str = Field(min_length=36, max_length=50)
    type: PersonalDataTypeEnum
    status: PersonalDataStatusEnum
    cancellation_details: PersonalDataDetails
    created_at: datetime
    expires_at: datetime | None = None
    metadata: dict[str, Any] | None = None

    model_config = ConfigDict(use_enum_values=True)



from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.sbp_banks_response import SbpBankListResponse


class SbpBanks:
    """
    Класс, представляющий модель SbpBanks.
    """

    base_path = "/sbp_banks"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def list(cls) -> SbpBankListResponse:
        """
        Возвращает список участников СБП

        :return: Объект ответа SbpBankListResponse, возвращаемого API при запросе списка участников СБП
        """
        instance = cls()

        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)

        return SbpBankListResponse(**response)



from datetime import datetime

from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.deal_enums import DealStatusEnum
from async_yookassa.models.deal_request import DealRequest
from async_yookassa.models.payment_submodels.amount import Amount


class DealResponse(DealRequest):
    id: str
    balance: Amount
    payout_balance: Amount
    status: DealStatusEnum
    created_at: datetime
    expires_at: datetime
    test: bool

    model_config = ConfigDict(use_enum_values=True)


class DealListResponse(BaseModel):
    type: str
    items: list[DealResponse]
    next_cursor: str | None = None



from typing import Any

from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.deal import DealBase
from async_yookassa.models.payout_submodels.payout_destination_data import (
    PayoutDestination,
)
from async_yookassa.models.payout_submodels.receipt_data import ReceiptData


class PayoutRequest(BaseModel):
    amount: Amount
    payout_destination_data: PayoutDestination | None = None
    payout_token: str | None = None
    payment_method_id: str | None = None
    description: str | None = Field(max_length=128, default=None)
    deal: DealBase | None = None
    self_employed: DealBase | None = None
    receipt_data: ReceiptData | None = None
    personal_data: list[DealBase] | None = None
    metadata: dict[str, Any] | None = None



from async_yookassa.configuration import Configuration
from async_yookassa.deal import Deal
from async_yookassa.invoice import Invoice
from async_yookassa.payment import Payment
from async_yookassa.payout import Payout
from async_yookassa.personal_data import PersonalData
from async_yookassa.refund import Refund
from async_yookassa.receipt import Receipt
from async_yookassa.sbp_banks import SbpBanks
from async_yookassa.self_employed import SelfEmployed
from async_yookassa.settings import Settings
from async_yookassa.webhooks import Webhook

__author__ = "Ivan Ashikhmin and YooMoney"
__email__ = "sushkoos@gmail.com and cms@yoomoney.ru"
__version__ = "0.5.2"
__all__ = [
    "Configuration",
    "Payment",
    "Invoice",
    "Refund",
    "Receipt",
    "Payout",
    "SelfEmployed",
    "SbpBanks",
    "PersonalData",
    "Deal",
    "Webhook",
    "Settings",
]



from pydantic import BaseModel

from async_yookassa.models.payment_submodels.airline import Airline
from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.deal import Deal
from async_yookassa.models.payment_submodels.receipt import Receipt
from async_yookassa.models.payment_submodels.transfers import Transfer


class CapturePaymentRequest(BaseModel):
    amount: Amount | None = None
    receipt: Receipt | None = None
    airline: Airline | None = None
    transfers: list[Transfer] | None = None
    deal: Deal | None = None



from pydantic import BaseModel, Field


class SbpBank(BaseModel):
    bank_id: str = Field(max_length=12)
    name: str = Field(max_length=128)
    bic: str = Field(max_length=9)


class SbpBankListResponse(BaseModel):
    type: str
    items: list[SbpBank]
    next_cursor: str | None = None



import re

from pydantic import BaseModel, ConfigDict, Field, field_validator

from async_yookassa.enums.card import CardTypeEnum, SourceEnum
from async_yookassa.models.payment_submodels.payment_method_submodels.card_product import (
    CardProduct,
)


class CardBase(BaseModel):
    number: str


class CardRequest(CardBase):
    expiry_year: str
    expiry_month: str
    cardholder: str | None = None
    csc: str | None = None


class CardResponse(BaseModel):
    first6: str | None = None
    last4: str
    expiry_year: str
    expiry_month: str
    card_type: CardTypeEnum
    card_product: CardProduct | None = None
    issuer_country: str | None = Field(min_length=2, max_length=2, default=None)
    issuer_name: str | None = None
    source: SourceEnum | None = None

    model_config = ConfigDict(use_enum_values=True)

    @field_validator("first6", mode="before")
    def validate_first6(cls, value: str) -> str:
        if not re.match("^[0-9]{6}$", value):
            raise ValueError("Invalid first6 value")

        return value

    @field_validator("last4", mode="before")
    def validate_last4(cls, value: str) -> str:
        if not re.match(r"^\d{4}$", value):
            raise ValueError("Invalid last4 value")

        return value

    @field_validator("expiry_year", mode="before")
    def validate_expiry_year(cls, value: str) -> str:
        if not re.match(r"^\d\d\d\d$", value) and 2000 < int(value) < 2200:
            raise ValueError("Invalid card expiry year value")

        return value

    @field_validator("expiry_month", mode="before")
    def validate_expiry_month(cls, value: str) -> str:
        if not re.match(r"^\d\d$", value) and 0 < int(value) <= 12:
            raise ValueError("Invalid card expiry month value")

        return value



from enum import Enum


class PartyEnum(str, Enum):
    merchant = "merchant"
    payment_network = "payment_network"
    yoo_money = "yoo_money"


class ReasonEnum(str, Enum):
    general_decline = "general_decline"
    insufficient_funds = "insufficient_funds"
    three_d_secure_failed = "3d_secure_failed"
    call_issuer = "call_issuer"
    canceled_by_merchant = "canceled_by_merchant"
    card_expired = "card_expired"
    country_forbidden = "country_forbidden"
    deal_expired = "deal_expired"
    expired_on_capture = "expired_on_capture"
    expired_on_confirmation = "expired_on_confirmation"
    fraud_suspected = "fraud_suspected"
    identification_required = "identification_required"
    internal_timeout = "internal_timeout"
    invalid_card_number = "invalid_card_number"
    invalid_csc = "invalid_csc"
    issuer_unavailable = "issuer_unavailable"
    payment_method_limit_exceeded = "payment_method_limit_exceeded"
    payment_method_restricted = "payment_method_restricted"
    permission_revoked = "permission_revoked"
    unsupported_mobile_operator = "unsupported_mobile_operator"
    rejected_by_payee = "rejected_by_payee"
    rejected_by_timeout = "rejected_by_timeout"
    yoo_money_account_closed = "yoo_money_account_closed"
    payment_article_number_not_found = "payment_article_number_not_found"
    payment_basket_id_not_found = "payment_basket_id_not_found"
    payment_tru_code_not_found = "payment_tru_code_not_found"
    some_articles_already_refunded = "some_articles_already_refunded"
    too_many_refunding_articles = "too_many_refunding_articles"
    one_time_limit_exceeded = "one_time_limit_exceeded"
    periodic_limit_exceeded = "periodic_limit_exceeded"
    recipient_check_failed = "recipient_check_failed"
    recipient_not_found = "recipient_not_found"


class PersonalDataReasonEnum(str, Enum):
    expired_by_timeout = "expired_by_timeout"



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.deal_submodels.settlements import (
    Settlement,
)


class DealBase(BaseModel):
    id: str = Field(min_length=36, max_length=50)


class Deal(DealBase):
    settlements: list[Settlement]


class DealRefund(DealBase):
    refund_settlements: list[Settlement]



from base64 import b64encode
from typing import Any

from httpx import AsyncClient, Response

from async_yookassa import Configuration
from async_yookassa.exceptions.api_error import APIError
from async_yookassa.exceptions.bad_request_error import BadRequestError
from async_yookassa.exceptions.forbidden_error import ForbiddenError
from async_yookassa.exceptions.not_found_error import NotFoundError
from async_yookassa.exceptions.response_processing_error import ResponseProcessingError
from async_yookassa.exceptions.too_many_request_error import TooManyRequestsError
from async_yookassa.exceptions.unauthorized_error import UnauthorizedError
from async_yookassa.models.deal_request import DealRequest
from async_yookassa.models.invoice_request import InvoiceRequest
from async_yookassa.models.payment_request import PaymentRequest
from async_yookassa.models.payout_request import PayoutRequest
from async_yookassa.models.personal_data_request import PersonalDataRequest
from async_yookassa.models.receipt_request import ReceiptRequest
from async_yookassa.models.refund_request import RefundRequest
from async_yookassa.models.self_employed_request import SelfEmployedRequest
from async_yookassa.models.user_agent import UserAgent
from async_yookassa.models.webhook_request import WebhookRequest


class APIClient:
    """
    Класс клиента API.
    """

    def __init__(self):
        self.configuration = Configuration.instantiate()
        self.endpoint = Configuration.api_endpoint()
        self.account_id = self.configuration.account_id
        self.secret_key = self.configuration.secret_key
        self.auth_token = self.configuration.auth_token
        self.async_client = AsyncClient(timeout=self.configuration.timeout)

        self.user_agent = UserAgent()
        if self.configuration.agent_framework:
            self.user_agent.framework = self.configuration.agent_framework
        if self.configuration.agent_cms:
            self.user_agent.cms = self.configuration.agent_cms
        if self.configuration.agent_module:
            self.user_agent.module = self.configuration.agent_module

    async def request(
        self,
        body: (
            PaymentRequest
            | InvoiceRequest
            | RefundRequest
            | ReceiptRequest
            | PayoutRequest
            | SelfEmployedRequest
            | PersonalDataRequest
            | DealRequest
            | WebhookRequest
            | None
        ) = None,
        method: str = "",
        path: str = "",
        query_params: dict[str, str] | None = None,
        headers: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """
        Подготовка запроса.

        :param method: HTTP метод
        :param path: URL запроса
        :param query_params: Словарь GET параметров запроса
        :param headers: Словарь заголовков запроса
        :param body: Тело запроса
        :return: Ответ на запрос в формате JSON
        """
        if isinstance(body, PaymentRequest):
            body = body.model_dump()

        request_headers = self.prepare_request_headers(headers=headers)
        raw_response = await self.execute(
            body=body, method=method, path=path, query_params=query_params, request_headers=request_headers
        )

        if raw_response.status_code != 200:
            self.__handle_error(raw_response=raw_response)

        return raw_response.json()

    async def execute(
        self,
        method: str,
        path: str,
        request_headers: dict[str, str],
        query_params: dict[str, str] | None = None,
        body: dict[str, Any] | None = None,
    ) -> Response:
        """
        Выполнение запроса.

        :param body: Тело запроса
        :param method: HTTP метод
        :param path: URL запроса
        :param query_params: Массив GET параметров запроса
        :param request_headers: Массив заголовков запроса
        :return: Response
        """
        async with self.async_client as client:
            # self.log_request(body, method, path, query_params, request_headers)

            raw_response = await client.request(
                method=method, url=self.endpoint + path, params=query_params, headers=request_headers, json=body
            )

            # self.log_response(raw_response.content, self.get_response_info(raw_response), raw_response.headers)

        return raw_response

    def prepare_request_headers(self, headers: dict[str, str]) -> dict[str, str]:
        """
        Подготовка заголовков запроса.

        :param headers: Словарь заголовков запроса
        :return: Словарь заголовков запроса
        """
        request_headers = {"Content-type": "application/json"}
        if self.auth_token is not None:
            auth_headers = {"Authorization": "Bearer " + self.auth_token}
        else:
            auth_headers = {"Authorization": self.basic_auth(self.account_id, self.secret_key)}

        request_headers.update(auth_headers)

        request_headers.update({"YM-User-Agent": self.user_agent.get_header_string()})

        if isinstance(headers, dict):
            request_headers.update(headers)
        return request_headers

    @staticmethod
    def basic_auth(username: str, password: str) -> str:
        """
        Формирование токена для Basic авторизации.

        :param username: Идентификатор аккаунта
        :param password: Секретный ключ магазина
        :return: Строка авторизации
        """
        token = b64encode(f"{username}:{password}".encode()).decode("ascii")
        return f"Basic {token}"

    @staticmethod
    def __handle_error(raw_response: Response) -> None:
        """
        Выбрасывает исключение по коду ошибки.
        """
        http_code = raw_response.status_code
        if http_code == BadRequestError.HTTP_CODE:
            raise BadRequestError(raw_response.json())
        elif http_code == ForbiddenError.HTTP_CODE:
            raise ForbiddenError(raw_response.json())
        elif http_code == NotFoundError.HTTP_CODE:
            raise NotFoundError(raw_response.json())
        elif http_code == TooManyRequestsError.HTTP_CODE:
            raise TooManyRequestsError(raw_response.json())
        elif http_code == UnauthorizedError.HTTP_CODE:
            raise UnauthorizedError(raw_response.json())
        elif http_code == ResponseProcessingError.HTTP_CODE:
            raise ResponseProcessingError(raw_response.json())
        else:
            raise APIError(raw_response.text)



from enum import Enum


class DealTypeEnum(str, Enum):
    safe_deal = "safe_deal"


class DealFeeMomentEnum(str, Enum):
    payment_succeeded = "payment_succeeded"
    deal_closed = "deal_closed"


class DealStatusEnum(str, Enum):
    opened = "opened"
    closed = "closed"



from enum import Enum


class SettlementTypeEnum(str, Enum):
    payout = "payout"


class SettlementReceiptEnum(str, Enum):
    cashless = "cashless"
    prepayment = "prepayment"
    postpayment = "postpayment"
    consideration = "consideration"



import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.self_employed_request import SelfEmployedRequest
from async_yookassa.models.self_employed_response import SelfEmployedResponse
from async_yookassa.utils import get_base_headers


class SelfEmployed:
    """
    Класс, представляющий модель SelfEmployed.
    """

    base_path = "/self_employed"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def find_one(cls, self_employed_id: str) -> SelfEmployedResponse:
        """
        Возвращает информацию о сомозанятом

        :param self_employed_id: Уникальный идентификатор сомозанятого
        :return: Объект ответа SelfEmployedResponse, возвращаемого API при запросе информации о сомозанятом
        """
        instance = cls()

        if not isinstance(self_employed_id, str):
            raise ValueError("Invalid self_employed_id value")

        path = instance.base_path + "/" + self_employed_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)
        return SelfEmployedResponse(**response)

    @classmethod
    async def create(
        cls, params: dict[str, Any] | SelfEmployedRequest, idempotency_key: uuid.UUID | None = None
    ) -> SelfEmployedResponse:
        """
        Создание сомозанятого

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа SelfEmployedResponse, возвращаемого API при запросе информации о сомозанятом
        """
        instance = cls()

        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = SelfEmployedRequest(**params)
        elif isinstance(params, SelfEmployedRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )

        return SelfEmployedResponse(**response)



from typing import Any

from pydantic import BaseModel, model_validator

from async_yookassa.models.payment_submodels.confirmation import (
    ConfirmationSelfEmployed,
)


class SelfEmployedRequest(BaseModel):
    itn: str | None = None
    phone: str | None = None
    confirmation: ConfirmationSelfEmployed | None = None

    @model_validator(mode="before")
    def validate_itn(cls, value: dict[str, Any]) -> dict[str, Any]:
        if value.get("itn") is None and value.get("phone") is None:
            raise ValueError("Both itn and phone values are empty in self_employed_request")

        return value



from enum import Enum


class PersonalDataTypeEnum(str, Enum):
    sbp_payout_recipient = "sbp_payout_recipient"
    payout_statement_recipient = "payout_statement_recipient"


class PersonalDataStatusEnum(str, Enum):
    waiting_for_operation = "waiting_for_operation"
    active = "active"
    canceled = "canceled"



from async_yookassa.exceptions.api_error import APIError


class TooManyRequestsError(APIError):
    """
    Превышен лимит запросов в единицу времени. Попробуйте снизить интенсивность запросов.
    """

    HTTP_CODE = 429



from typing import Any

from pydantic import BaseModel, ConfigDict, Field, model_validator

from async_yookassa.enums.payout import PayoutTypeEnum
from async_yookassa.models.payment_submodels.payment_method_submodels.card import (
    CardBase,
)


class PayoutDestinationBase(BaseModel):
    type: PayoutTypeEnum
    card: CardBase | None = None
    bank_id: str | None = Field(max_length=12, default=None)
    phone: str | None = None
    account_number: str | None = Field(min_length=11, max_length=33, default=None)

    model_config = ConfigDict(use_enum_values=True)


class PayoutDestination(PayoutDestinationBase):

    @model_validator(mode="before")
    def validate_required_fields(cls, values: dict[str, Any]) -> dict[str, Any]:
        type_value = values.get("type")

        if type_value == PayoutTypeEnum.bank_card:
            if not values.get("card"):
                raise ValueError("Field 'card' is required for type 'bank_card'")

        elif type_value == PayoutTypeEnum.sbp:
            if not values.get("bank_id"):
                raise ValueError("Field 'bank_id' are required for type 'sbp'")
            if not values.get("phone"):
                raise ValueError("Field 'phone' are required for type 'sbp'")

        elif type_value == PayoutTypeEnum.yoo_money:
            if not values.get("account_number"):
                raise ValueError("Field 'account_number' is required for type 'yoo_money'")

        return values


class PayoutDestinationResponse(PayoutDestinationBase):
    recipient_checked: bool | None = None

    @model_validator(mode="before")
    def validate_required_fields(cls, values: dict[str, Any]) -> dict[str, Any]:
        type_value = values.get("type")

        if type_value == PayoutTypeEnum.sbp:
            if not values.get("bank_id"):
                raise ValueError("Field 'bank_id' are required for type 'sbp'")
            if not values.get("phone"):
                raise ValueError("Field 'phone' are required for type 'sbp'")
            if not values.get("recipient_checked"):
                raise ValueError("Field 'recipient_checked' are required for type 'sbp'")

        elif type_value == PayoutTypeEnum.yoo_money:
            if not values.get("account_number"):
                raise ValueError("Field 'account_number' is required for type 'yoo_money'")

        return values



from pydantic import BaseModel, EmailStr, Field


class Customer(BaseModel):
    full_name: str | None = Field(max_length=256, default=None)
    inn: str | None = Field(max_length=12, default=None)
    email: EmailStr | None = None
    phone: str | None = Field(max_length=11, min_length=11, default=None)



import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.deal_request import DealRequest
from async_yookassa.models.deal_response import DealListResponse, DealResponse
from async_yookassa.utils import get_base_headers


class Deal:
    """
    Класс, представляющий модель Deal.
    """

    base_path = "/deals"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def find_one(cls, deal_id: str) -> DealResponse:
        """
        Возвращает информацию о сделке

        :param deal_id: Уникальный идентификатор сделки
        :return: Объект ответа DealResponse, возвращаемого API при запросе сделки
        """
        instance = cls()

        if not isinstance(deal_id, str):
            raise ValueError("Invalid payment_id value")

        path = instance.base_path + "/" + deal_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)
        return DealResponse(**response)

    @classmethod
    async def create(
        cls, params: dict[str, Any] | DealRequest, idempotency_key: uuid.UUID | None = None
    ) -> DealResponse:
        """
        Создание сделки

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа DealResponse, возвращаемого API при запросе сделки
        """
        instance = cls()

        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = DealRequest(**params)
        elif isinstance(params, DealRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )
        return DealResponse(**response)

    @classmethod
    async def list(cls, params: dict[str, str] | None = None) -> DealListResponse:
        """
        Возвращает список сделок

        :param params: Данные передаваемые в API
        :return: Объект ответа DealListResponse, возвращаемого API при запросе списка сделок
        """

        instance = cls()

        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path, query_params=params)
        return DealListResponse(**response)



from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.cancellation_details import (
    PartyEnum,
    PersonalDataReasonEnum,
    ReasonEnum,
)


class Details(BaseModel):
    model_config = ConfigDict(use_enum_values=True)


class CancellationDetails(Details):
    party: PartyEnum
    reason: ReasonEnum


class RefundDetails(Details):
    party: PartyEnum
    reason: ReasonEnum


class PayoutDetails(Details):
    party: PartyEnum
    reason: ReasonEnum


class PersonalDataDetails(Details):
    party: PartyEnum
    reason: PersonalDataReasonEnum



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.payment_response import ReceiptRegistrationEnum
from async_yookassa.models.payment_submodels.confirmation import (
    ConfirmationSelfEmployedResponse,
)


class SelfEmployedResponse(BaseModel):
    id: str = Field(min_length=36, max_length=50)
    status: ReceiptRegistrationEnum
    created_at: datetime
    itn: str | None = None
    phone: str | None = None
    confirmation: ConfirmationSelfEmployedResponse | None = None
    test: bool

    model_config = ConfigDict(use_enum_values=True)
